/***************************************************
*  File			: datum.c
*  Autor 		: D.Nolte
*  Date 		: 12.04.2005
*  Version		: 0.0.9
*  -------------------------------------------------
*  Discription	:
*  
*  
*  -------------------------------------------------
*  Changed :
*  Date / Autor / comment.
*
****************************************************/
#include <avr/io.h>   		//AVR register defination
#include <avr/interrupt.h>
#include <stdlib.h>
#include <math.h>

#ifndef _motor_H_
	#include "motor.h"		//Motor Steuerung
#endif

#ifndef _main_H_			//Main Aufruf
	#include "main.h"		
#endif

#ifndef _datum_H_
	#include "datum.h"		//Kanlender berechung und Datum/Zeit
#endif


//Globale Struktur fr Datum Uhrzeit
DateTime_t DateAndTime;

char monthvalue []={0,3,3,6,1,4,6,2,5,0,3,5};
char month[]={31,28,31,30,31,30,31,31,30,31,30,31};

/*************************************************************************
* This function controll of leap year
* the return value is 
* 0 = not leap year
* 1 = leap year
*************************************************************************/
char leapyear (int year){
 if (!(year%100)) return (char)(year%400);
    else return (char)(year%4);
}
/*************************************************************************
* This function controll of leap year
* the return value is 
* 0 = not leap year
* 1 = leap year
*************************************************************************/
static inline char leap_year (void){
 if (!(DateAndTime.year%100)) return (char)(DateAndTime.year%400);
    else return (char)(DateAndTime.year%4);
}
/*************************************************************************
* This function compute the week day
* Return vaule ist the count of the days 
* MO = 0
*************************************************************************/
/*char week_day(char day,char month,int year){
 int j = year - 1900;
 int v = j / 4;
 if (leapyear(year)==0) ++day;
 int tday = j + v + monthvalue[month-1]+ (day - 1); //- leapyear(j);
 int bday = tday /7;
 return (char) tday - (bday * 7);
}*/

/*************************************************************************
* This function compute the week day
* Return vaule ist the count of the days 
* MO = 0
*************************************************************************/
char week_day_f(){
 int j = DateAndTime.year - 1900;
 int v = j / 4;
 //if (leap_year(DateAndTime.year)==0) ++DateAndTime.day;
 if (leap_year()==0) ++DateAndTime.day;
 int tday = j + v + monthvalue[DateAndTime.month-1]+ (DateAndTime.day - 1); //- leapyear(j);
 int bday = tday /7;
 return (char) tday - (bday * 7);
}

/*************************************************************************
* Time Interrupt for clock
*************************************************************************/
ISR (TIMER1_OVF_vect){ 
	TCNT1 = 65535 - (Quartz / 1024);  //65535 - 15625
	cli();
    if (++DateAndTime.second==60){
        DateAndTime.second=0;
        if (++DateAndTime.minute==60){
            DateAndTime.minute=0;
            if (++DateAndTime.hour==24){
                DateAndTime.hour=0;
                if (++DateAndTime.day==32){
                    DateAndTime.month++;
                    DateAndTime.day=1;
                }
                else if (DateAndTime.day==31){                    
                    if ((DateAndTime.month==4) || (DateAndTime.month==6) || (DateAndTime.month==9) || (DateAndTime.month==11)){
                        DateAndTime.month++;
                        DateAndTime.day=1;
                    }
                }
                else if (DateAndTime.day==30){
                    if(DateAndTime.month==2){
                       DateAndTime.month++;
                       DateAndTime.day=1;
                    }
                }              
                else if (DateAndTime.day==29){
                    if((DateAndTime.month==2) && (leapyear(DateAndTime.year))){
                        DateAndTime.month++;
                        DateAndTime.day=1;
                    }                
                }                          
                if (DateAndTime.month==13){
                    DateAndTime.month=1;
                    DateAndTime.year++;
                }
            }
        }
    }  
	sei();
}  
/*************************************************************************
* Init clock
*************************************************************************/
void Clock_init (void){	
	DateAndTime.second=11;
	DateAndTime.minute=10;
	DateAndTime.hour=14;
	DateAndTime.day=12;
	DateAndTime.month=3;
	DateAndTime.year=2006;
	DateAndTime.weekday = week_day_f();	//Aktualisieren 
	TIMSK = _BV(TOIE1);					// Enable time interrupt	
	TCCR1B |= (1<<CS10 | 0<<CS11 | 1<<CS12); 	//Set prescal value
	//Berechnung 1 sec Takt
	//     1      |        1       |        1           |     1	 |
	//------------| ---------------|--------------------|--------|
	//         1  |           1    |                    |		 |15625
	// CS * ------| 1024 * --------| 1024 * 0,0000000625|0,000064|
	//      QUARTZ|        16000000|                    |		 |
	TCNT1 = 65535 - (Quartz / 1024);  //65535 - 15625
}
//----------------------------------------------------------------------------
// Vergleicht die Schaltzeiten mit der Aktuellen Zeit
//----------------------------------------------------------------------------
char GRT_DT(int count,char cmd){
//char GRT_DT(char cmd){

    DateAndTime.weekday = week_day_f();//Aktualisieren
	
	//if (DateAndTime.minute >= dt.minute && DateAndTime.hour >= dt.hour && DateAndTime.weekday == dt.weekday) return 1;
	if (PM[count].active != 0x01){
		if ((DateAndTime.weekday == PM[count].day) && (DateAndTime.minute >= PM[count].start_M) && (DateAndTime.hour >= PM[count].start_H)){
			PM[count].active = 0x01;		//Setze Aktive
			cmd = 0x01;						//Command = 0x01 (ffnen)
			return cmd;
		}
	}
	else if (PM[count].active == 0x01){
		if ((DateAndTime.weekday == PM[count].day) && (DateAndTime.minute >= PM[count].end_M) && (DateAndTime.hour >= PM[count].end_H)){
			PM[count].active = 0x00;		//Rcksetze Aktive
			cmd= 0x02;						//Command = 0x02 (Schliessen)
			return cmd;
		}
	}
	return cmd;//nichts passiert ;-)
}
//----------------------------------------------------------------------------
// Lese Uhrzeit und Datum
//----------------------------------------------------------------------------
DateTime_t get_DT(void){
	//Aktualisieren damit nicht im Interrupt ntig
	DateAndTime.weekday = week_day_f();
    return DateAndTime;
}
//----------------------------------------------------------------------------
// Setze Uhrzeit und Datum
//----------------------------------------------------------------------------
void set_DT(DateTime_t DT){
    cli(); //Interrupt sperren beim Setzen des Datum und Uhrzeit
	DateAndTime = DT;
	DateAndTime.weekday = week_day_f();
	sei();
}


