/***************************************************
*  File			: lcd.c
*  Autor 		: D.Nolte
*  Date 		: 11.04.2005
*  Version		: 0.0.9
*  -------------------------------------------------
*  Discription	:
*  
*  
*  -------------------------------------------------
*  Changed :
*  Date / Autor / comment.
*
*  Hardware
*  2 X 16 Zeichen MPU KS0066/HD44780
****************************************************/

#include <avr/io.h>   	//AVR
#include <avr/interrupt.h>

#ifndef _lcd_H_				//LCD-Display ansteuerung
	#include "lcd.h"
#endif

#ifndef _delays_h_			//Zeitverzgerungen
	#include "delays.h"
#endif


/** Sendet ein Kommando ans LCD ******************************************
*************************************************************************/

void LCD_cmd(char cmd){
	cli();
	LCD_CMD &=0b00011111;	//Reset E/WR/RS
	LCD_DATA =cmd;
	LCD_CMD |=0b00100000;	//Set E
	LCD_CMD &=0b11011111;	//Reset E
	LCD_CMD |=0b10000000;	//Set RS	
	sei();
	delay_ms(1); 			//Verzgerung immer 4 ms anstelle busy abfragen
}

/** Init LCD *************************************************************
*	Initialized LCD Display												 *
*------------------------------------------------------------------------*
*	1.	Wait 15 mili second												 *
*	2-4.	Send init sequence											 *
*	5.	Send 001110xxb	0x38h	Display init			  				 *
*		        |||														 *
*			    ||(1=5x10 0=5x7 font)									 *
*			    |(1=2line 0=1line Display)								 *
*			    (1=8bit 0=4bit Data)									 *
*	6.	Send 00001101b	0x0Dh	Display on				    			 *
*			      |||													 *
*			      ||(1=Sign under Cursor flash 0=flashing off)			 *
*			      |(1=Cursor not visible 0=Cursor visible)				 *
*			      (1=Display on 0=Display off							 *
*	7.	Send 00000001b	0x01h	Display clear							 *
*	8.	Send 00000110b  0x06h	Entry Mode								 *
*			       ||													 *
*			       |(1=Display shift 0=Display no shift					 *
*			       (1=Addresspointer inc 0=Addresspointer dec			 *
*************************************************************************/
void LCD_init(void){
  delay_ms(15);
  LCD_cmd(0x30);
  LCD_cmd(0x30);
  LCD_cmd(0x30);	//Init sequenc 
  LCD_cmd(0x38);	//Zeilenanzahl + Font
  LCD_cmd(0x0D);	//Display AN
  LCD_cmd(LCD_clr);	//Display lschen
  LCD_cmd(0x6);		//Entry mode
}
/** LCD Lschen **********************************************************
*************************************************************************/
void LCD_clear(void){ 
    LCD_cmd(LCD_clr);
}
/** Setzt den Cursor an position *****************************************
*************************************************************************/
void LCD_cursor(char line,char colum){ 
   if (line ==LCD_line1) LCD_cmd(0x80+colum);
   if (line ==LCD_line2) LCD_cmd(0xC0+colum);
}
/** Send Charactar to LCD **************************************************
*************************************************************************/
void LCD_char(char string){
   LCD_CMD &=0b10111111;
   LCD_DATA=string;
   LCD_CMD |=0b00100000;
   LCD_CMD &=0b11011111;
   LCD_CMD |=0b10000000;
   delay_ms(1);
}
/** Print String on LCD **************************************************
*************************************************************************/
void LCD_print(char line,char colum,char string[15]){
  int i=0;				//Schleifenzhler fr Zeichenkette
  
  LCD_cursor(line,colum);   //Setzt den Cursor an Anfang der angegebenen Zeile
  while ((i<16) && (string[i] != 0X00 )){     //Gibt die Zeichenkette aus
     LCD_char(string[i++]);
  }
}
/** Print String on LCD **************************************************
// fr pointer test
*************************************************************************/
void LCD_sprint(char line,char colum,char *string){
  int i=0;				//Schleifenzhler fr Zeichenkette
  
  LCD_cursor(line,colum);   //Setzt den Cursor an Anfang der angegebenen Zeile
  
  while ((i<16) && (string[i] != 0X00 )){     //Gibt die Zeichenkette aus
     LCD_char(string[i++]);
  }
}
