/***************************************************
*  Datei		: main.c
*  Autor 		: D.Nolte
*  Datum		: 11.04.2005
*  Version		: 0.0.1
*  -------------------------------------------------
*  Beschreibung	:
*  Hauptfunktion der Steuerung
*  
*  -------------------------------------------------
*  Aenderungen :
*  Datum / Autor / Bemerkung.
*
* Hardware info
* CPU			: ATmega32
* clockcycle	: 16 MHz
* Display		: 8 Bit-Mode LCD 2*16 Zeichen
* Drive			: 2 DC Motor H-Brcke
*
*  
* todo
* 
*
****************************************************/

#include <avr/io.h>   		//AVR
#include <avr/interrupt.h>
#include <stdlib.h>

#ifndef _delays_h_			//Zeitverzgerungen
	#include "delays.h"
#endif

#ifndef _lcd_H_				//LCD-Display Ansteuerung
	#include "lcd.h"
#endif

#ifndef _key_H_				//Tastenabfrage
	#include "key.h"
#endif

#ifndef _datum_H_
	#include "datum.h"		//Kanlender berechung und Datum/Zeit
#endif

#ifndef _menue_H			//Bedienmenue
	#include "menue.h"
#endif

#ifndef _debug_H_			//Debug Anzeige
	#include "debug.h"
#endif
	
#ifndef _motor_H_
	#include "motor.h"		//Motor Steuerung
#endif

#ifndef _USART_H_
	#include "uart.h" 		//RS232 functions
#endif

#ifndef _main_H_			//Main aufruf
	#include "main.h"		
#endif

//----------------------------------------------------------------------------
//
//
//----------------------------------------------------------------------------
int init(void){
	
	DDRA=0x03;	//reservierd fr Analog input 
	DDRB=0xF8;  //bit 0-2 Button Input / bit 5-7 LCD -Command Output
	DDRC=0xFF;	//Data out LCD-Display
	DDRD=0x3c;	//Motor and RS232
	PORTA &=0xFc;//pull-up ein
	PORTB &=0x07;//1F
	PORTC &=0xFF;//reset all outut
	PORTD &=0xc0;
	//Init LCD-Display
	LCD_init();
	//Init Uart
	usart_init(103);
	//Init clock fr Datum and Zeit
	Clock_init();
	
	//Noch hier spter in init 
	for (int i=0;i< (Anzahl_Programme+1);i++){
		PM[i].nr = i;
		PM[i].day=0x0;
		PM[i].start_H=14;
		PM[i].start_M=12;
		PM[i].end_H=14;
		PM[i].end_M=13;
	};
	
	//Noch hier spter in Init
	MH.Act_key = 0;
	MH.Act_menue = 0;
	MH.active = 0;
	MH.List_count = 1;
	MH.List_index = 1;
	MH.start=0;
	MH.Auto_count=1;
	MH.old_second=0;
	MH.released=0;
	
	sei(); // Enalbe all interrupts
	
	return 0;
}
//----------------------------------------------------------------------------
//
//
//----------------------------------------------------------------------------
int main(void){
    int i;
	int command=0;			//0x01=ffnen 0x02=schliessen
	int command_new=0;
	int info=0;
   	init();
	LCD_clear();
	LCD_print(LCD_line1,0x00,"Zeitschaltuhr");
	LCD_print(LCD_line2,0x00,"SW - 0.0.6");
	delay_s(2);
	while (1){
		i =0;
		//Einlesen Men Tasten
		MH.Act_key = get_keycode();
		
		//Men aufrufen
		Menue();

		//Bedienelement Tasten einlesen
		command_new = usart_getc_nowait();
		
		//Taster Auswerten vielleicht spter als funktion
		if (command_new != -1) {
			command = 0x00;
			if ((command_new & 0x80)) command = 0x04;  //STOP
			if ((command_new & 0x40)) command = 0x03;	//Auf
			if ((command_new & 0x20)) command = 0x02;	//Zu
			if ((command_new & 0x10)) command = 0x01;	//Automatik
		}
		//sprintf(debug1,"->%2d =%2d",command_new,command);
		
		/*if (Handle.start && (i++ < Anzahl_Programme+1)) {
			command = GRT_DT(&Prog[i],command);
		}
		else i = 1;*/
		
	// Motoransteuerung 
		Open_w(command);
		
		//PORTA &= ~0x03; //Debug
   }
   return (1);
}

SIGNAL(__vector_default)
{
	asm volatile("nop");
	return; 
}
