/***************************************************
*  File			: uart.c
*  Autor 		: D.Nolte
*  Date 		: 11.04.2005
*  Version		: 0.0.1
*  -------------------------------------------------
*  Discription	:
*  
*  
*  -------------------------------------------------
*  Changed :
*  Date / Autor / comment.
*
****************************************************/


#include <avr/io.h>   	//AVR
#include <avr/interrupt.h>
#include <compat/ina90.h>
#include <stdlib.h>

#ifndef _USART_H_
	#include "uart.h" 		//RS232 functions
#endif

#ifndef _debug_H_
	#include "debug.h"
#endif

#ifndef _FIFO_H_
	#include "fifo.h"
#endif

/* Static Variables */

#define BUFSIZE_IN 0x20
uint8_t inbuf[BUFSIZE_IN];
fifo_t infifo;

#define BUFSIZE_OUT 0x20
uint8_t outbuf[BUFSIZE_OUT];
fifo_t outfifo;

//---------------------------------------------------------------------------------------
// Initialize USART
//---------------------------------------------------------------------------------------
 
void usart_init( unsigned int baudrate )
{
	// Set the baud rate
	UBRRH = (unsigned char) (baudrate>>8);                  
	UBRRL = (unsigned char) baudrate;
	
	// Enable UART receiver(Interrupt) and transmitter 
	UCSRA = (1<<RXC);
	UCSRB = (1<<RXCIE) | (1<<RXEN);
	//UCSRB = (1<<RXEN);  
	
	// Set frame format: 8N1
	UCSRC = (1<<URSEL)|(1<<UCSZ1)|(1<<UCSZ0);
	// Flush receive buffer

	do
	{
		uint8_t dummy;
		(void) (dummy = UDR);
	}
	while (UCSRA & (1 << RXC));
	
	// FIFOs fr Ein- und Ausgabe initialisieren
	fifo_init (&infifo, inbuf, BUFSIZE_IN);
	fifo_init (&outfifo, outbuf, BUFSIZE_OUT);
}

//---------------------------------------------------------------------------------------
// Interrupt routinen
//---------------------------------------------------------------------------------------

// Empfangene Zeichen werden in die Eingabgs-FIFO gespeichert und warten dort
SIGNAL (SIG_UART_RECV)
{
	PORTA |= 0x01;
	_inline_fifo_put (&infifo, UDR);
	PORTA &= ~0x01;
}

// Ein Zeichen aus der Ausgabe-FIFO lesen und ausgeben
// Ist das Zeichen fertig ausgegeben, wird ein neuer SIG_UART_DATA-IRQ getriggert
// Ist die FIFO leer, deaktiviert die ISR ihren eigenen IRQ.
SIGNAL (SIG_UART_DATA)
{
	if (outfifo.count > 0)
		UDR = _inline_fifo_get (&outfifo);
	else
		UCSRB &= ~(1 << UDRIE);
}


//---------------------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------------------
int usart_putc (const uint8_t c)
{
	int ret = fifo_put (&outfifo, c);
	UCSRB |= (1 << UDRIE);
	return ret;
}

int usart_getc_nowait ()
{
	return fifo_get_nowait (&infifo);
}

uint8_t usart_getc_wait ()
{
	return fifo_get_wait (&infifo);
}
